@@ -69,6 +69,9 @@ bekk_fit <- function(spec, data, QML_t_ratios = FALSE,
   if (any(is.na(data))) {
     stop("\nNAs in data.\n")
   }
+  if (is.null(ncol(data))) {
+    stop("The data matrix should contain at least two variables.")
+  }
   if (ncol(data) < 2) {
     stop("The data matrix should contain at least two variables.")
   }
@@ -184,9 +187,13 @@ bekk_fit.bekk <- function(spec, data, QML_t_ratios = FALSE,
 
   if (inherits(data, "ts")) {
     sigma_t <- ts(sigma_t, start = time(data)[1], frequency = frequency(data))
+    H_t <- ts(var_process$sigma_t, start = time(data)[1], frequency = frequency(data))
   }
   else if(inherits(data, "xts") || inherits(data, "zoo") ){
     sigma_t <- xts(sigma_t, order.by = time(data))
+    H_t <-  xts(var_process$sigma_t, order.by = time(data))
+  }else{
+    H_t <-  var_process$sigma_t
   }
 
   # Final check if BEKK is valid
@@ -205,7 +212,7 @@ bekk_fit.bekk <- function(spec, data, QML_t_ratios = FALSE,
                  log_likelihood = params$likelihood,
                  BEKK_valid = BEKK_valid,
                  sigma_t = sigma_t,
-                 H_t = var_process$sigma_t,
+                 H_t = H_t,
                  e_t = var_process$e_t,
                  Second_moments_of_residuals = cov(var_process$e_t),
                  iter = params$iter,
@@ -333,9 +340,13 @@ bekk_fit.bekka <- function(spec, data, QML_t_ratios = FALSE,
 
   if (inherits(data, "ts")) {
     sigma_t <- ts(sigma_t, start = time(data)[1], frequency = frequency(data))
+    H_t <- ts(var_process$sigma_t, start = time(data)[1], frequency = frequency(data))
   }
   else if(inherits(data, "xts") || inherits(data, "zoo") ){
     sigma_t <- xts(sigma_t, order.by = time(data))
+    H_t <-  xts(var_process$sigma_t, order.by = time(data))
+  }else{
+    H_t <-  var_process$sigma_t
   }
 
   # Final check if BEKK is valid
@@ -358,7 +369,7 @@ bekk_fit.bekka <- function(spec, data, QML_t_ratios = FALSE,
                  log_likelihood = params$likelihood,
                  BEKK_valid = BEKK_valid,
                  sigma_t = sigma_t,
-                 H_t = var_process$sigma_t,
+                 H_t = H_t,
                  e_t = var_process$e_t,
                  Second_moments_of_residuals = cov(var_process$e_t),
                  iter = params$iter,
@@ -446,9 +457,13 @@ bekk_fit.dbekk <- function(spec, data, QML_t_ratios = FALSE,
 
   if (inherits(data, "ts")) {
     sigma_t <- ts(sigma_t, start = time(data)[1], frequency = frequency(data))
+    H_t <- ts(var_process$sigma_t, start = time(data)[1], frequency = frequency(data))
   }
   else if(inherits(data, "xts") || inherits(data, "zoo") ){
     sigma_t <- xts(sigma_t, order.by = time(data))
+    H_t <-  xts(var_process$sigma_t, order.by = time(data))
+  }else{
+    H_t <-  var_process$sigma_t
   }
 
   # Final check if BEKK is valid
@@ -467,7 +482,7 @@ bekk_fit.dbekk <- function(spec, data, QML_t_ratios = FALSE,
                  log_likelihood = params$likelihood,
                  BEKK_valid = BEKK_valid,
                  sigma_t = sigma_t,
-                 H_t = var_process$sigma_t,
+                 H_t = H_t,
                  e_t = var_process$e_t,
                  Second_moments_of_residuals = cov(var_process$e_t),
                  iter = params$iter,
@@ -558,9 +573,13 @@ bekk_fit.dbekka <- function(spec, data, QML_t_ratios = FALSE,
 
   if (inherits(data, "ts")) {
     sigma_t <- ts(sigma_t, start = time(data)[1], frequency = frequency(data))
+    H_t <- ts(var_process$sigma_t, start = time(data)[1], frequency = frequency(data))
   }
   else if(inherits(data, "xts") || inherits(data, "zoo") ){
     sigma_t <- xts(sigma_t, order.by = time(data))
+    H_t <-  xts(var_process$sigma_t, order.by = time(data))
+  }else{
+    H_t <-  var_process$sigma_t
   }
 
   # Final check if BEKK is valid
@@ -583,7 +602,7 @@ bekk_fit.dbekka <- function(spec, data, QML_t_ratios = FALSE,
                  log_likelihood = params$likelihood,
                  BEKK_valid = BEKK_valid,
                  sigma_t = sigma_t,
-                 H_t = var_process$sigma_t,
+                 H_t = H_t,
                  e_t = var_process$e_t,
                  Second_moments_of_residuals = cov(var_process$e_t),
                  iter = params$iter,
@@ -671,9 +690,13 @@ bekk_fit.sbekk <- function(spec, data, QML_t_ratios = FALSE,
 
   if (inherits(data, "ts")) {
     sigma_t <- ts(sigma_t, start = time(data)[1], frequency = frequency(data))
+    H_t <- ts(var_process$sigma_t, start = time(data)[1], frequency = frequency(data))
   }
   else if(inherits(data, "xts") || inherits(data, "zoo") ){
     sigma_t <- xts(sigma_t, order.by = time(data))
+    H_t <-  xts(var_process$sigma_t, order.by = time(data))
+  }else{
+    H_t <-  var_process$sigma_t
   }
 
   # Final check if BEKK is valid
@@ -692,7 +715,7 @@ bekk_fit.sbekk <- function(spec, data, QML_t_ratios = FALSE,
                  log_likelihood = params$likelihood,
                  BEKK_valid = BEKK_valid,
                  sigma_t = sigma_t,
-                 H_t = var_process$sigma_t,
+                 H_t = H_t,
                  e_t = var_process$e_t,
                  Second_moments_of_residuals = cov(var_process$e_t),
                  iter = params$iter,
@@ -783,9 +806,13 @@ bekk_fit.sbekka <- function(spec, data, QML_t_ratios = FALSE,
 
   if (inherits(data, "ts")) {
     sigma_t <- ts(sigma_t, start = time(data)[1], frequency = frequency(data))
+    H_t <- ts(var_process$sigma_t, start = time(data)[1], frequency = frequency(data))
   }
   else if(inherits(data, "xts") || inherits(data, "zoo") ){
     sigma_t <- xts(sigma_t, order.by = time(data))
+    H_t <-  xts(var_process$sigma_t, order.by = time(data))
+  }else{
+    H_t <-  var_process$sigma_t
   }
 
   # Final check if BEKK is valid
@@ -808,7 +835,7 @@ bekk_fit.sbekka <- function(spec, data, QML_t_ratios = FALSE,
                  log_likelihood = params$likelihood,
                  BEKK_valid = BEKK_valid,
                  sigma_t = sigma_t,
-                 H_t = var_process$sigma_t,
+                 H_t = H_t,
                  e_t = var_process$e_t,
                  Second_moments_of_residuals = cov(var_process$e_t),
                  iter = params$iter,
